@@ -30,7 +30,7 @@ use DynaLoader ();
 use Carp;
 
 use vars   qw( $VERSION @ISA );
-$VERSION = "0.67";
+$VERSION = "0.69";
 @ISA     = qw( DynaLoader );
 bootstrap Text::CSV_XS $VERSION;
 
@@ -78,6 +78,7 @@ my %def_attr = (
     _ERROR_INPUT	=> undef,
     _COLUMN_NAMES	=> undef,
     _BOUND_COLUMNS	=> undef,
+    _AHEAD		=> undef,
     );
 my $last_new_err = Text::CSV_XS->SetDiag (0);
 
@@ -116,7 +117,8 @@ sub new
     $self;
     } # new
 
-my %_cache_id = (	# Keep in sync with XS!
+# Keep in sync with XS!
+my %_cache_id = ( # Only expose what is accessed from within PM
     quote_char		=>  0,
     escape_char		=>  1,
     sep_char		=>  2,
@@ -128,15 +130,10 @@ my %_cache_id = (	# Keep in sync with XS!
     allow_double_quoted	=>  8,
     allow_whitespace	=>  9,
     blank_is_undef	=> 10,
-
     eol			=> 11,	# 11 .. 18
-    eol_len		=> 19,
-    eol_is_cr		=> 20,
-    has_types		=> 21,
     verbatim		=> 22,
     empty_is_undef	=> 23,
     auto_diag		=> 24,
-
     _is_bound		=> 25,	# 25 .. 28
     );
 
@@ -147,10 +144,7 @@ sub _set_attr_C
     defined $val or $val = 0;
     $] >= 5.008002 and utf8::decode ($val);
     $self->{$name} = $val;
-    $self->{_CACHE} or return;
-    my @cache = unpack "C*", $self->{_CACHE};
-    $cache[$_cache_id{$name}] = unpack "C", $val;
-    $self->{_CACHE} = pack "C*", @cache;
+    $self->_cache_set ($_cache_id{$name}, $val);
     } # _set_attr_C
 
 # A flag
@@ -159,22 +153,15 @@ sub _set_attr_X
     my ($self, $name, $val) = @_;
     defined $val or $val = 0;
     $self->{$name} = $val;
-    $self->{_CACHE} or return;
-    my @cache = unpack "C*", $self->{_CACHE};
-    $cache[$_cache_id{$name}] = 0 + $val;
-    $self->{_CACHE} = pack "C*", @cache;
-    } # _set_attr_C
+    $self->_cache_set ($_cache_id{$name}, 0 + $val);
+    } # _set_attr_X
 
 # A number
 sub _set_attr_N
 {
     my ($self, $name, $val) = @_;
     $self->{$name} = $val;
-    $self->{_CACHE} or return;
-    my @cache = unpack "C*", $self->{_CACHE};
-    my $i = $_cache_id{$name};
-    $cache[$i++] = $_ for unpack "C*", pack "N", $val;
-    $self->{_CACHE} = pack "C*", @cache;
+    $self->_cache_set ($_cache_id{$name}, 0 + $val);
     } # _set_attr_N
 
 # Accessor methods.
@@ -216,19 +203,8 @@ sub eol
     if (@_) {
 	my $eol = shift;
 	defined $eol or $eol = "";
-	my $eol_len = length $eol;
 	$self->{eol} = $eol;
-	$self->{_CACHE} or return;
-	my @cache = unpack "C*", $self->{_CACHE};
-	if (($cache[$_cache_id{eol_len}] = $eol_len) < 8) {
-	    $cache[$_cache_id{eol_is_cr}] = $eol eq "\r" ? 1 : 0;
-	    }
-	else {
-	    $cache[$_cache_id{eol_is_cr}] = 0;
-	    }
-	$eol .= "\0\0\0\0\0\0\0\0";
-	$cache[$_cache_id{eol} + $_] = unpack "C", substr $eol, $_, 1 for 0 .. 7;
-	$self->{_CACHE} = pack "C*", @cache;
+	$self->_cache_set ($_cache_id{eol}, $eol);
 	}
     $self->{eol};
     } # eol
@@ -299,7 +275,7 @@ sub empty_is_undef
 sub verbatim
 {
     my $self = shift;
-    @_ and $self->_set_attr_C ("verbatim", shift);
+    @_ and $self->_set_attr_X ("verbatim", shift);
     $self->{verbatim};
     } # verbatim
 
@@ -355,10 +331,25 @@ sub error_diag
 	}
 
     my $context = wantarray;
-    unless (defined $context) {	# Void context
-	if ($diag[0]) {
+    unless (defined $context) {	# Void context, auto-diag
+	if ($diag[0] && $diag[0] != 2012 && $self && ref $self) {
 	    my $msg = "# CSV_XS ERROR: $diag[0] - $diag[1]\n";
-	    $self && ref $self && $self->{auto_diag} > 1 ? die $msg : warn $msg;
+
+	    my $lvl = $self->{auto_diag};
+	    if ($lvl < 2) {
+		my @c = caller (2);
+		if (@c >= 11 && $c[10] && ref $c[10] eq "HASH") {
+		    my $hints = $c[10];
+		    (exists $hints->{autodie} && $hints->{autodie} or
+		     exists $hints->{"guard Fatal"} &&
+		    !exists $hints->{"no Fatal"}) and
+			$lvl++;
+		    # Future releases of autodie will probably set $^H{autodie}
+		    #  to "autodie @args", like "autodie :all" or "autodie open"
+		    #  so we can/should check for "open" or "new"
+		    }
+		}
+	    $lvl > 1 ? die $msg : warn $msg;
 	    }
 	return;
 	}
@@ -556,31 +547,21 @@ Text::CSV_XS - comma-separated values manipulation routines
 
  use Text::CSV_XS;
 
- $csv = Text::CSV_XS->new ();          # create a new object
- $csv = Text::CSV_XS->new (\%attr);    # create a new object
+ my @rows;
+ my $csv = Text::CSV_XS->new ({ binary => 1 }) or
+     die "Cannot use CSV: ".Text::CSV->error_diag ();
+ open my $fh, "<:encoding(utf8)", "test.csv" or die "test.csv: $!";
+ while (my $row = $csv->getline ($fh)) {
+     $row->[2] =~ m/pattern/ or next; # 3rd field should match
+     push @rows, $row;
+     }
+ $csv->eof or $csv->error_diag ();
+ close $fh;
 
- $status  = $csv->combine (@columns);  # combine columns into a string
- $line    = $csv->string ();           # get the combined string
-
- $status  = $csv->parse ($line);       # parse a CSV string into fields
- @columns = $csv->fields ();           # get the parsed fields
-
- $status       = $csv->status ();      # get the most recent status
- $bad_argument = $csv->error_input (); # get the most recent bad argument
- $diag         = $csv->error_diag ();  # if an error occured, explains WHY
-
- $status = $csv->print ($io, $colref); # Write an array of fields
-                                       # immediately to a file $io
- $colref = $csv->getline ($io);        # Read a line from file $io,
-                                       # parse it and return an array
-                                       # ref of fields
- $csv->bind_columns (@refs);           # Set return fields for getline ()
- $csv->column_names (@names);          # Set column names for getline_hr ()
- $ref = $csv->getline_hr ($io);        # getline (), but returns a hashref
- $eof = $csv->eof ();                  # Indicate if last parse or
-                                       # getline () hit End Of File
-
- $csv->types (\@t_array);              # Set column types
+ $csv->eol ("\r\n");
+ open $fh, ">:encoding(utf8)", "new.csv" or die "new.csv: $!";
+ $csv->print ($fh, $_) for @rows;
+ close $fh or die "new.csv: $!";
 
 =head1 DESCRIPTION
 
@@ -786,7 +767,7 @@ So lines like:
 are now correctly parsed, even though it violates the CSV specs.
 
 Note that B<all> whitespace is stripped from start and end of each
-field. That would make is more a I<feature> than a way to be able
+field. That would make it more a I<feature> than a way to be able
 to parse bad CSV lines, as
 
  1,   2.0,  3,   ape  , monkey
@@ -968,12 +949,14 @@ anymore, and getline () chomps line endings on reading.
 Set to true will cause C<error_diag ()> to be automatically be called
 in void context upon errors.
 
+In case of error C<2012 - EOF>), this call will be void.
+
 If set to a value greater than 1, it will die on errors instead of
 warn.
 
-Future extensions to this feature will include auto-detection of the
-C<autodie> module being enabled, which will raise the value of C<auto_diag>
-with C<1> on the moment the error is detected.
+Future extensions to this feature will include more reliable auto-detection
+of the C<autodie> module being enabled, which will raise the value of
+C<auto_diag> with C<1> on the moment the error is detected.
 
 =back
 
@@ -1021,26 +1004,15 @@ C<error_diag ()> will return a string like
 
  "INI - Unknown attribute 'ecs_char'"
 
-=head2 combine
-
- $status = $csv->combine (@columns);
-
-This object function constructs a CSV string from the arguments, returning
-success or failure.  Failure can result from lack of arguments or an argument
-containing an invalid character.  Upon success, C<string ()> can be called to
-retrieve the resultant CSV string.  Upon failure, the value returned by
-C<string ()> is undefined and C<error_input ()> can be called to retrieve an
-invalid argument.
-
 =head2 print
 
  $status = $csv->print ($io, $colref);
 
-Similar to combine, but it expects an array ref as input (not an array!)
-and the resulting string is not really created, but immediately written
-to the I<$io> object, typically an IO handle or any other object that
-offers a I<print> method. Note, this implies that the following is wrong
-in perl 5.005_xx and older:
+Similar to C<combine () + string () + print>, but more efficient. It
+expects an array ref as input (not an array!) and the resulting string is
+not really created, but immediately written to the I<$io> object, typically
+an IO handle or any other object that offers a I<print> method. Note, this
+implies that the following is wrong in perl 5.005_xx and older:
 
  open FILE, ">", "whatever";
  $status = $csv->print (\*FILE, $colref);
@@ -1055,6 +1027,17 @@ In particular the I<$csv-E<gt>string ()>, I<$csv-E<gt>status ()>,
 I<$csv->fields ()> and I<$csv-E<gt>error_input ()> methods are meaningless
 after executing this method.
 
+=head2 combine
+
+ $status = $csv->combine (@columns);
+
+This object function constructs a CSV string from the arguments, returning
+success or failure.  Failure can result from lack of arguments or an argument
+containing an invalid character.  Upon success, C<string ()> can be called to
+retrieve the resultant CSV string.  Upon failure, the value returned by
+C<string ()> is undefined and C<error_input ()> can be called to retrieve an
+invalid argument.
+
 =head2 string
 
  $line = $csv->string ();
@@ -1062,20 +1045,6 @@ after executing this method.
 This object function returns the input to C<parse ()> or the resultant CSV
 string of C<combine ()>, whichever was called more recently.
 
-=head2 parse
-
- $status = $csv->parse ($line);
-
-This object function decomposes a CSV string into fields, returning
-success or failure.  Failure can result from a lack of argument or the
-given CSV string is improperly formatted.  Upon success, C<fields ()> can
-be called to retrieve the decomposed fields .  Upon failure, the value
-returned by C<fields ()> is undefined and C<error_input ()> can be called
-to retrieve the invalid argument.
-
-You may use the I<types ()> method for setting column types. See the
-description below.
-
 =head2 getline
 
  $colref = $csv->getline ($io);
@@ -1091,6 +1060,20 @@ reference to an empty list.
 The I<$csv-E<gt>string ()>, I<$csv-E<gt>fields ()> and I<$csv-E<gt>status ()>
 methods are meaningless, again.
 
+=head2 parse
+
+ $status = $csv->parse ($line);
+
+This object function decomposes a CSV string into fields, returning
+success or failure.  Failure can result from a lack of argument or the
+given CSV string is improperly formatted.  Upon success, C<fields ()> can
+be called to retrieve the decomposed fields .  Upon failure, the value
+returned by C<fields ()> is undefined and C<error_input ()> can be called
+to retrieve the invalid argument.
+
+You may use the I<types ()> method for setting column types. See the
+description below.
+
 =head2 getline_hr
 
 The C<getline_hr ()> and C<column_names ()> methods work together to allow
@@ -1370,7 +1353,9 @@ or using the slower C<combine ()> and C<string ()> methods:
   close $csv_fh or die "hello.csv: $!";
 
 For more extended examples, see the C<examples/> subdirectory in the
-original distribution. The following files can be found there:
+original distribution or the git repository at
+http://repo.or.cz/w/Text-CSV_XS.git?a=tree;f=examples. The following files
+can be found there:
 
 =over 2
 
@@ -1396,6 +1381,14 @@ A script to convert CSV to Microsoft Excel. This requires L<Date::Calc>
 and L<Spreadsheet::WriteExcel>. The converter accepts various options and
 can produce UTF-8 Excel files.
 
+=item csvdiff
+
+A script that provides colorized diff on sorted CSV files, assuming first
+line is header and first field is the key. Output options include colorized
+ANSI escape codes or HTML.
+
+  $ csvdiff --html --output=diff.html file1.csv file2.csv
+
 =back
 
 =head1 CAVEATS
@@ -1505,9 +1498,9 @@ If the constructor failed, the cause can be found using C<error_diag ()> as a
 class method, like C<Text::CSV_XS->error_diag ()>.
 
 C<$csv->error_diag ()> is automatically called upon error when the contractor
-was called with C<auto_diag> set to 1 or 2, or when C<autodie> is in effect
-(NYI).  When set to 1, this will cause a C<warn ()> with the error message,
-when set to 2, it will C<die ()>.
+was called with C<auto_diag> set to 1 or 2, or when C<autodie> is in effect.
+When set to 1, this will cause a C<warn ()> with the error message, when set
+to 2, it will C<die ()>. C<2012 - EOF> is excluded from C<auto_diag> reports.
 
 Currently errors as described below are available. I've tried to make the error
 itself explanatory enough, but more descriptions will be added. For most of
@@ -1586,7 +1579,7 @@ has been selected with the constructor.
 Sequences like C<1,"foo\rbar",2> are only allowed when the binary option
 has been selected with the constructor.
 
-=item 2023 "EIQ - QUO character not allowed
+=item 2023 "EIQ - QUO character not allowed"
 
 Sequences like C<"foo "bar" baz",quux> and C<2023,",2008-04-05,"Foo, Bar",\n>
 will cause this error.
@@ -1644,7 +1637,7 @@ exhausted before the quote is found, that field is not terminated.
 
 =item 3007 "EHR - bind_columns needs refs to writeable scalars"
 
-=item 3008 "EHR - unexpected error in bound fields
+=item 3008 "EHR - unexpected error in bound fields"
 
 =back
 
